function vol_sort=wavelet_denoise(Y,level,display)

% X=wavelet_denoise(Y,level)
%
% Funcio que treu el soroll blanc d'una imatge a nivell de gris (basat en
% S. Grace Chang et al., "Adaptive Wavelet Thresholding for Image Denoising
% and Compresion).
%
% INPUT:
%
% Y = Imatge d'entrada.
% level = Nivell de descomposicio (per defecte 5).
%
% OUTPUT:
%
% X = Imatge sense soroll

%% Inicialitzacions
if ~exist('level','var'),
    level=5;
end
if ~exist('display','var'),
    display=false;
end



%% Per totes les llesques

for ll=1:size(Y,3)

    %%Wavelet transform
    % Calculem la transformada de wavelet i els subbands
    [C,S] = wavedec2(double(Y(:,:,ll)),level,'db4');
    for k=1:level,
        HH(k).c=detcoef2('d',C,S,k);
        HL(k).c=detcoef2('h',C,S,k);
        LH(k).c=detcoef2('v',C,S,k);
    end

    LL(level).c=appcoef2(C,S,'db4',level);

    %% Thresholding
    % Calculem la desviacio del soroll
    [n m]=size(HH(1).c);
    sigma_soroll=median(abs(reshape(HH(1).c,n*m,1)))/0.675;

    % Calculem el threshold de cada subband, filtrem i reconstruim un nivell
    for i=level:-1:1,

        % Reconstruim un nivell
        if i<level,
            [nx ny] = size(HH(i+1).c);
            LL(i).c=idwt2(LL(i+1).c(1:nx,1:ny),HL(i+1).c,LH(i+1).c,HH(i+1).c,'db4');
        end

        % Filtrem la diagonal
        [n m]=size(HH(i).c);
        sigma_y_2=(1/(n*m))*sum(sum(HH(i).c.^2));
        sigma_x=sqrt(max(sigma_y_2-sigma_soroll,0));
        T=(sigma_soroll^2)/sigma_x;
        HH(i).c=sign(HH(i).c).*max(abs(HH(i).c)-T,0);

        % Filtrem la horitzontal
        sigma_y_2=(1/(n*m))*sum(sum(HL(i).c.^2));
        sigma_x=sqrt(max(sigma_y_2-sigma_soroll,0));
        T=(sigma_soroll^2)/sigma_x;
        HL(i).c=sign(HL(i).c).*max(abs(HL(i).c)-T,0);

        % Filtrem la vertical
        sigma_y_2=(1/(n*m))*sum(sum(LH(i).c.^2));
        sigma_x=sqrt(max(sigma_y_2-sigma_soroll,0));
        T=(sigma_soroll^2)/sigma_x;
        LH(i).c=sign(LH(i).c).*max(abs(LH(i).c)-T,0);

    end

    if ll==91
        bu=1;
    end
    
    %% Wavelet inverse
    [nx ny]=size(HL(1).c);
    % Reconstruim la imatge final
    X=idwt2(LL(1).c(1:nx,1:ny),HL(1).c,LH(1).c,HH(1).c,'db4');
    [nx ny]=size(Y(:,:,ll));
    X=X(1:nx,1:ny);
    X=uint16(X-min(min(X)));

    %% Results
%     if display,
%         figure
%         subplot(1,3,1)
%         plotDicom(Y);
%         title('MRI original');
%         subplot(1,3,2)
%         plotDicom(X);
%         title('MRI sense soroll');
%         subplot(1,3,3)
%         plotDicom(Y-X);
%         title('Diferencia entre la original i la filtrada');
%     end

vol_sort(:,:,ll)=X;
end

end
